/* Copyright (C) 2011, 2012, 2013 by Bundesanstalt für Gewässerkunde
 * Software engineering by Intevation GmbH
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */

package org.dive4elements.river.artifacts.model;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

import org.dive4elements.artifacts.Artifact;
import org.dive4elements.artifacts.CallContext;
import org.dive4elements.artifacts.DataProvider;

import org.dive4elements.artifactdatabase.state.DefaultFacet;

import org.dive4elements.river.artifacts.MainValuesArtifact;
import org.dive4elements.river.artifacts.math.Linear;
import org.dive4elements.river.jfree.RiverAnnotation;
import org.dive4elements.river.jfree.StickyAxisAnnotation;
import org.dive4elements.river.exports.fixings.FixChartGenerator;


/**
 * Facet to show Main W Values.
 */
public class MainValuesWFacet
extends      DefaultFacet
implements   FacetTypes {

    /** Own logger. */
    private static Logger logger = Logger.getLogger(MainValuesWFacet.class);

    /** Do we want MainValues at Gauge (not interpolated)? */
    protected boolean isAtGauge;

    /** Trivial Constructor. */
    public MainValuesWFacet(String name, String description, boolean atGauge) {
        this.description = description;
        this.name = name;
        this.index = 0;
        this.isAtGauge = atGauge;
    }


    /**
     * Set the hit-point in W where a line drawn from the axis would hit the
     * curve in WQDay (if hit).
     * Employ linear interpolation.
     */
    protected static void setHitPoint(WQDay wqday, StickyAxisAnnotation annotation) {
        int idx = 0;
        float w = annotation.getPos();
        boolean wIncreases = wqday.getW(0) < wqday.getW(wqday.size()-1);
        if (wIncreases) {
            while (idx < wqday.size() && wqday.getW(idx) < w) {
                idx++;
            }
        }
        else {
            idx = wqday.size() -1;
            while (idx > 0 && wqday.getW(idx) > w) {
                idx--;
            }
        }

        double day = 0d;
        int mod = (wIncreases) ? -1 : +1;
        if (idx != 0 && idx < wqday.size()-1-mod) {
            day = Linear.linear(w, wqday.getW(idx +mod), wqday.getW(idx),
                wqday.getDay(idx+mod), wqday.getDay(idx));
            annotation.setHitPoint((float)day);
        }
        else {
            logger.debug("StickyAnnotation does not hit wqday curve");
        }
    }


    /**
     * Returns the data this facet provides.
     *
     * @param artifact the owner artifact.
     * @param context  the CallContext (can be used to find out if in
     *                 navigable fixation-setting, or durationcurve).
     *
     * @return the data.
     */
    @Override
    public Object getData(Artifact artifact, CallContext context) {
        MainValuesArtifact mvArtifact = (MainValuesArtifact) artifact;

        List<NamedDouble> ws = mvArtifact.getMainValuesW(isAtGauge);
        List<StickyAxisAnnotation> xy = new ArrayList<StickyAxisAnnotation>();

        // Find whether a duration curve is on the blackboard.
        WQDay wqdays = null;
        List<DataProvider> providers = context.
            getDataProvider(DurationCurveFacet.BB_DURATIONCURVE);
        if (providers.size() < 1) {
            logger.warn("Could not find durationcurve data provider.");
            // Do we have a current km in context?
            // If so, we are likely fetching data for a navigable
            // diagram (i.e. in fixation branch).
            if (context.getContextValue(FixChartGenerator.CURRENT_KM) != null) {
                Double ckm = (Double) context.getContextValue(FixChartGenerator.CURRENT_KM);
                // Return linearly interpolated values, in m if not at gauge,
                // in cm if at gauge.
                ws = mvArtifact.getMainValuesW(new double[] {ckm});
            }
        }
        else {
            wqdays = (WQDay) providers.get(0).provideData(
                DurationCurveFacet.BB_DURATIONCURVE,
                null,
                context);
        }

        for (NamedDouble w: ws) {
            StickyAxisAnnotation annotation =
                new StickyAxisAnnotation(
                    w.getName(),
                    (float) w.getValue(),
                    StickyAxisAnnotation.SimpleAxis.Y_AXIS);
            xy.add(annotation);
            if (wqdays != null) {
                setHitPoint(wqdays, annotation);
            }
        }

        return new RiverAnnotation(description, xy);
    }


    /**
     * Create a deep copy of this Facet.
     * @return a deep copy.
     */
    @Override
    public MainValuesWFacet deepCopy() {
        MainValuesWFacet copy = new MainValuesWFacet(this.name,
            description, this.isAtGauge);
        copy.set(this);
        return copy;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
